#pragma once
//#include "resource.h"       // main symbols

//#include "..\idl\netex.h"
//#include "net.h"
//#include "PortParser.h"
//#include "iptraits.h"

// CAddressTrans
enum
{
	PROTOCOL_TCP,
	PROTOCOL_UDP,
	PROTOCOL_IPraw,
	PROTOCOL_IPmulticast,
	PROTOCOL_IP,

	PROTOCOL_MAX
};

struct CSimpleStream
{
	virtual void Append(TCHAR c) = 0;

	CSimpleStream& operator << (TCHAR c)
	{
		Append(c);
		return *this;
	}

	CSimpleStream& operator << (LPCTSTR s)
	{
		while (*s)
			Append(*s++);
		return *this;
	}
};

class ATL_NO_VTABLE CAddressTrans :
	public CComObjectRootEx<CComMultiThreadModel>,
	public CComCoClass<CAddressTrans, &CLSID_AddressTrans>,
	public IAddressTrans
{
public:
	CAddressTrans()
	{
		//if(!nRefCount)
		//{
			PortParser.Parse(g_StringPool[IDS_STRING1027]);
			IpAddrCache.Start();
		//}

		++nRefCount;
	}

	~CAddressTrans()
	{
		if(!nRefCount)
			IpAddrCache.Stop();
	}

DECLARE_REGISTRY_RESOURCEID(IDR_ADDRESSTRANS)

DECLARE_NOT_AGGREGATABLE(CAddressTrans)

BEGIN_COM_MAP(CAddressTrans)
	COM_INTERFACE_ENTRY(IAddressTrans)
END_COM_MAP()



	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}

	void FinalRelease()
	{
	}

	static UINT ProtocolName[PROTOCOL_MAX];		 // Array of names of supported protocols
	static UINT ProtocolDesc[PROTOCOL_MAX];		 // Array of descriptions of supported protocols
	static UINT ProtocolDeviceName[PROTOCOL_MAX]; // Array of device names of supported protocols
	static ULONG ConnectionType[PROTOCOL_MAX];

	void GetLocAddressesTree(
						UINT nDeviceIndex,
						CSimpleStream &out,
						FILTER_RULE   *rule,
						LPCTSTR		  process,
						const TRANSPORT_ADDRESS *laddr,
						const TRANSPORT_ADDRESS *raddr,
						UINT count);

	void GetRemAddressesTree(
						UINT nDeviceIndex,
						CSimpleStream &out,
						FILTER_RULE   *rule,
						LPCTSTR		  process,
						const TRANSPORT_ADDRESS *laddr,
						const TRANSPORT_ADDRESS *raddr,
						UINT count);

	void GetLocProcessesTree(
						UINT nDeviceIndex,
						CSimpleStream &out,
						FILTER_RULE   *rule,
						LPCTSTR		  process,
						const TRANSPORT_ADDRESS *laddr,
						const TRANSPORT_ADDRESS *raddr,
						UINT count);
public:
	void Initialize(UINT index)
	{
		//ProtocolIndex = index;
		

	}

	// ITdiProtocol
public:
	STDMETHOD_(LPCTSTR, GetProtocolDeviceName)(UINT i);    // Returns a device's name
    STDMETHOD_(LPCTSTR, GetProtocolName)(UINT i);          // Returns a protocol's name
    STDMETHOD_(LPCTSTR, GetProtocolDesc)(UINT i);          // Returns a protocol's description
	STDMETHOD_(LPCTSTR, GetProtocolDriverName)(UINT i);	 // Returns a protocol's driver name

	STDMETHOD_(ULONG, GetConnectionType)(UINT i);			// Returns a protocol's driver name
	
    // Translates an address from string form to binary TDI form.
    // Returns a pointer to TRANSPORT_ADDRESS structure, the same as
    // transferred through parameter <address>, or NULL when some
    // error occured.
    // Parameters are:
    //      <str>       -   the address presented in the string form;
    //      <address>   -   the pointer to the buffer where the address should be stored;
    //      <size>      -   on input specifies the size in bytes of the buffer,
    //                      on output specifies the size of the address.
    STDMETHOD_(TRANSPORT_ADDRESS*, TranslateStringToAddress)(
                                    /*[in]*/		LPCTSTR str,
                                    /*[out]*/		TRANSPORT_ADDRESS *address,
                                    /*[in, out]*/	DWORD *size);

	/* Flags for TranslateAddressToString method
    enum
    {
        // string should be presented as address in digital form
        // (for example 212.5.70.42:23003)
        TF_ADDRESS  = 0x00000001,
        
        // string should be presented as address in named form
        // (for example www.examples.net:23003) 
        TF_NAMED    = 0x00000002,

        // method should include any additional
        // information about address
        TF_INFO     = 0x00000004,

		// method should refresh all information about address
		TF_REFRESH	= 0x00000008
    };
	*/


    // Translates an address from binary TDI form to string form.
    // Returns a string, the same as transferred through parameter
    // <str>, (LPTSTR) -1 when the method needs a long time to retrieve information
	// or NULL when some error occured.
    // Parameters are:
    //      <flags>         -   any number of flags TF_xxxx;
    //      <address>       -   the address presented in binary form;
    //      <str>           -   the pointer to the buffer where string should be stored;
    //      <size>          -   on input specifies the size in bytes of the buffer,
    //                          on output specifies the size of the string.
	//		<handler>		-	This handler should be called when the method returned (LPTSTR) 1 value
	//							and when the result of retrieving of addition information is known
	//							Handler should recall this method to retrieve additional information.
    STDMETHOD_(LPTSTR, TranslateAddressToString)(
							UINT nDeviceIndex,
                            DWORD flags,
                            /*[in]*/		const TRANSPORT_ADDRESS *address,
                            /*[out]*/		LPTSTR str,
                            /*[in, out]*/	DWORD *size,
                            /*[in]*/		UPDATEHANDLER handler = NULL);

	STDMETHOD_(BOOL, GetTreeItems)(
								UINT nDeviceIndex,
				/* [in]	*/		ULONG	treetype,
				/* [in, out] */	LPTSTR	buffer,
				/* [in, out] */	PFILTER_RULE rule,
				/* [in, out] */	DWORD   *buffersize,
				/* [in] */		LPCTSTR process,
				/* [in]	*/  	const TRANSPORT_ADDRESS *laddr,
				/* [in]	*/  	const TRANSPORT_ADDRESS *raddr,
				/* [in]    */    UINT count);

	

	inline LPTSTR __ResolvePortDecor(int protocol, USHORT port, LPTSTR str);

	static UINT nRefCount;		// to initialize statics only once in constructor

	static CPortParser			PortParser;
	static CIpAddrCache			IpAddrCache;

};

OBJECT_ENTRY_AUTO(__uuidof(AddressTrans), CAddressTrans)
